@@ -8,7 +8,7 @@
  *
  *    In some countries OX, OX Open-Xchange, open xchange and OXtender
  *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the Open-Xchange, Inc. group of companies.
+ *    trademarks.
  *    The use of the Logos is not covered by the GNU General Public License.
  *    Instead, you are allowed to use these Logos according to the terms and
  *    conditions of the Creative Commons License, Version 2.5, Attribution,
@@ -62,7 +62,6 @@ import java.util.LinkedHashMap;
 import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
-import java.util.Map.Entry;
 import org.apache.http.HttpException;
 import org.apache.http.HttpHost;
 import org.apache.http.HttpRequest;
@@ -96,8 +95,10 @@ import org.json.JSONInputStream;
 import org.json.JSONObject;
 import org.json.JSONValue;
 import com.openexchange.ajax.container.ThresholdFileHolder;
+import com.openexchange.antiabuse.AllowParameters;
 import com.openexchange.antiabuse.AntiAbuseService;
-import com.openexchange.antiabuse.ReportValue;
+import com.openexchange.antiabuse.Protocol;
+import com.openexchange.antiabuse.ReportParameters;
 import com.openexchange.antiabuse.Status;
 import com.openexchange.exception.OXException;
 import com.openexchange.java.Charsets;
@@ -156,6 +157,10 @@ public class WeakforcedClient implements AntiAbuseService {
 
     private static final String JSON_FIELD_STATUS = "status";
 
+    private static final String JSON_FIELD_MSG = "msg";
+
+    private static final String JSON_FIELD_ATTRIBUTES = "r_attrs";
+
     // -------------------------------------------------------------------------------------------------------------- //
 
     /** The status code policy to obey */
@@ -262,37 +267,55 @@ public class WeakforcedClient implements AntiAbuseService {
     }
 
     @Override
-    public Status allow(String login, String password, String remoteAddress, Map<String, String> attributes) throws OXException {
+    public Status allow(AllowParameters parameters) throws OXException {
         try {
-            Map<String, String> parameters = new LinkedHashMap<String, String>(2);
-            parameters.put("command", "allow");
+            Map<String, String> requestParameters = new LinkedHashMap<String, String>(2);
+            requestParameters.put("command", "allow");
 
             JSONObject jBody = new JSONObject(6);
-            jBody.put("login", login);
-            jBody.put("remote", remoteAddress);
-            jBody.put("pwhash", createPasswordHash(login, password, secret, algorithm, truncate));
+            jBody.put("login", parameters.getLogin());
+            jBody.put("remote", parameters.getRemoteAddress());
+            jBody.put("pwhash", createPasswordHash(parameters.getLogin(), parameters.getPassword(), secret, algorithm, truncate));
+            String userAgent = parameters.getUserAgent();
+            if (!Strings.isEmpty(userAgent)) {
+                jBody.put("device_id", userAgent);
+            }
+            Protocol protocol = parameters.getProtocol();
+            if (null != protocol) {
+                jBody.put("protocol", parameters.getProtocol().getName());
+            }
+            Map<String, String> attributes = parameters.getAttributes();
             if (null != attributes && !attributes.isEmpty()) {
                 jBody.put("attrs", new JSONObject(attributes));
             }
 
-            JSONObject jStatus = executePost(WeakforcedCall.ALLOW, parameters, jBody, JSON);
+            JSONObject jStatus = executePost(WeakforcedCall.ALLOW, requestParameters, jBody, JSON);
 
             int status = jStatus.getInt(JSON_FIELD_STATUS);
+            String message = null;
+            Map<String, Object> rAttributes = null;
             Map<String, String> properties = null;
             {
                 int length = jStatus.length();
                 if (length > 1) {
                     // Has more than one JSON field; check others...
                     properties = new LinkedHashMap<String, String>(length);
-                    for (Entry<String, Object> entry : jStatus.entrySet()) {
-                        if (!JSON_FIELD_STATUS.equals(entry.getKey())) {
+                    for (Map.Entry<String, Object> entry : jStatus.entrySet()) {
+                        String jName = entry.getKey();
+                        if (JSON_FIELD_STATUS.equals(jName)) {
+                            // Ignore
+                        } else if (JSON_FIELD_ATTRIBUTES.equals(jName)) {
+                            rAttributes = ((JSONObject) entry.getValue()).asMap();
+                        } else if (JSON_FIELD_MSG.equals(jName)) {
+                            message = entry.getValue().toString();
+                        } else {
                             properties.put(entry.getKey(), entry.getValue().toString());
                         }
                     }
                 }
             }
 
-            return new Status(status, properties);
+            return new Status(status, message, rAttributes, properties);
         } catch (JSONException e) {
             throw WeakforcedExceptionCodes.JSON_ERROR.create(e, e.getMessage());
         } catch (RuntimeException e) {
@@ -301,13 +324,13 @@ public class WeakforcedClient implements AntiAbuseService {
     }
 
     @Override
-    public void report(ReportValue reportValue, String login, String password, String remoteAddress) throws OXException {
+    public void report(ReportParameters parameters) throws OXException {
         try {
-            Map<String, String> parameters = new LinkedHashMap<String, String>(2);
-            parameters.put("command", "report");
+            Map<String, String> requestParameters = new LinkedHashMap<String, String>(2);
+            requestParameters.put("command", "report");
 
             JSONObject jBody = new JSONObject(8);
-            switch (reportValue) {
+            switch (parameters.getReportValue()) {
                 case DENIED_BY_SERVICE:
                     jBody.put("success",  false);
                     jBody.put("policy_reject", true);
@@ -322,11 +345,19 @@ public class WeakforcedClient implements AntiAbuseService {
                     jBody.put("success", true);
                     break;
             }
-            jBody.put("login", login);
-            jBody.put("remote", remoteAddress);
-            jBody.put("pwhash", createPasswordHash(login, password, secret, algorithm, truncate));
+            jBody.put("login", parameters.getLogin());
+            jBody.put("remote", parameters.getRemoteAddress());
+            jBody.put("pwhash", createPasswordHash(parameters.getLogin(), parameters.getPassword(), secret, algorithm, truncate));
+            String userAgent = parameters.getUserAgent();
+            if (!Strings.isEmpty(userAgent)) {
+                jBody.put("device_id", userAgent);
+            }
+            Protocol protocol = parameters.getProtocol();
+            if (null != protocol) {
+                jBody.put("protocol", parameters.getProtocol().getName());
+            }
 
-            executePost(WeakforcedCall.REPORT, parameters, jBody, VOID);
+            executePost(WeakforcedCall.REPORT, requestParameters, jBody, VOID);
         } catch (JSONException e) {
             throw WeakforcedExceptionCodes.JSON_ERROR.create(e, e.getMessage());
         } catch (RuntimeException e) {
